/* This file is Copyright 2002 Level Control Systems.  See the included LICENSE.txt file for details. */

#include "system/SetupSystem.h"

#ifdef WIN32
# include <winsock2.h>
# include <signal.h>
#else
# ifdef __BEOS__
#  include <signal.h>
# else
#  include <sys/signal.h>  // changed signal.h to sys/signal.h to work with OS/X
# endif
#endif

namespace muscle {

extern Mutex * _muscleLogLock;

#if defined(MUSCLE_USE_MUTEXES_FOR_ATOMIC_OPERATIONS)
const uint32 MUTEX_POOL_SIZE = 256;
static Mutex * _atomicMutexes = NULL;
#endif

ThreadSetupSystem :: ThreadSetupSystem()
{
   _muscleLogLock = &_logLock;
   
#if defined(MUSCLE_USE_MUTEXES_FOR_ATOMIC_OPERATIONS)
   _atomicMutexes = newnothrow Mutex[MUTEX_POOL_SIZE];
   MASSERT(_atomicMutexes, "Couldn't allocate atomic mutexes!");
#endif
}

ThreadSetupSystem :: ~ThreadSetupSystem()
{
#if defined(MUSCLE_USE_MUTEXES_FOR_ATOMIC_OPERATIONS)
   delete [] _atomicMutexes;
#endif
   _muscleLogLock = NULL;
}

#if defined(MUSCLE_USE_MUTEXES_FOR_ATOMIC_OPERATIONS)
int32 DoMutexAtomicIncrement(int32 * count, int32 delta)
{
   MASSERT(_atomicMutexes, "Please declare a SetupSystem object before doing any atomic incrementing!");
   Mutex & mutex = _atomicMutexes[(((uint32)count)/sizeof(int))%MUTEX_POOL_SIZE];
   (void) mutex.Lock();
   int32 ret = *count = (*count + delta);
   (void) mutex.Unlock();
   return ret;
}
#endif

NetworkSetupSystem :: NetworkSetupSystem()
{
#ifdef WIN32
   WORD versionWanted = MAKEWORD(1, 1); 
   WSADATA wsaData; 
   int ret = WSAStartup(versionWanted, &wsaData);
   MASSERT((ret == 0), "NetworkSetupSystem:  Couldn't initialize Winsock!");
#else
   signal(SIGPIPE, SIG_IGN);  // avoid evil SIGPIPE signals from sending on a closed socket
#endif
}

NetworkSetupSystem :: ~NetworkSetupSystem()
{
#ifdef WIN32
   WSACleanup();
#endif
}

};  // end muscle namespace
